-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation")

L:RegisterTranslations("enUS", function() return {
	["Resetting data as requested by %s."] = true,
	["Reset"] = true,
	["Options for when to reset the data."] = true,
	["Reset now"] = true,
	["Immediately resets your data."] = true,
	["Reset group"] = true,
	["Resets the Violation data for every group member. Only the group leader or an officer can do this."] = true,
	["Leader resets"] = true,
	["Accept reset requests from raid leaders or officers."] = true,
	["Auto-reset"] = true,
	["Entering combat"] = true,
	["Automatically reset the data when entering combat."] = true,
	["Leaving combat"] = true,
	["Automatically reset the data when leaving combat."] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------
local tablet = AceLibrary("Tablet-2.0")
local BC = AceLibrary("Babble-Class-2.2")
local roster = AceLibrary("Roster-2.1")

local playerName = nil

local coloredNames = setmetatable({}, {__index =
	function(self, key)
		local obj = roster:GetUnitObjectFromName(key)
		if not obj then return nil end
		self[key] = "|cff" .. BC:GetHexColor(obj.class) .. key .. "|r"
		return self[key]
	end
})

local data = nil
local dirty = nil
local db = nil

local _G = getfenv(0)

-----------------------------------------------------------------------
-- Options
-----------------------------------------------------------------------

local options = {
	type = "group",
	args = {
		reset = {
			type = "group",
			name = L["Reset"],
			desc = L["Options for when to reset the data."],
			args = {
				now = {
					type = "execute",
					name = L["Reset now"],
					desc = L["Immediately resets your data."],
					func = function() Violation:Reset() end,
					order = 100,
				},
				group = {
					type = "execute",
					name = L["Reset group"],
					desc = L["Resets the Violation data for every group member. Only the group leader or an officer can do this."],
					func = function() Violation:ResetGroup() end,
					disabled = function()
						return not IsRaidLeader() or not IsRaidOfficer() or not IsPartyLeader()
					end,
					order = 101,
				},
				leader = {
					type = "toggle",
					name = L["Leader resets"],
					desc = L["Accept reset requests from raid leaders or officers."],
					get = function() return Violation.db.profile.leaderReset end,
					set = function(v) Violation.db.profile.leaderReset = v end,
					order = 102,
				},
				spacer = {
					type = "header",
					name = " ",
					order = 150,
				},
				autoheader = {
					type = "header",
					name = L["Auto-reset"],
					order = 200,
				},
				gaincombat = {
					type = "toggle",
					name = L["Entering combat"],
					desc = L["Automatically reset the data when entering combat."],
					get = function() return Violation.db.profile.gainCombat end,
					set = function(v)
						if v then
							Violation:RegisterEvent("PLAYER_REGEN_DISABLED", "Reset")
						else
							Violation:UnregisterEvent("PLAYER_REGEN_DISABLED", "Reset")
						end
						Violation.db.profile.gainCombat = v
					end,
					order = 201,
				},
				leavecombat = {
					type = "toggle",
					name = L["Leaving combat"],
					desc = L["Automatically reset the data when leaving combat."],
					get = function() return Violation.db.profile.leaveCombat end,
					set = function(v)
						if v then
							Violation:RegisterEvent("PLAYER_REGEN_ENABLED", "Reset")
						else
							Violation:UnregisterEvent("PLAYER_REGEN_ENABLED", "Reset")
						end
						Violation.db.profile.leaveCombat = v
					end,
					order = 202,
				},
			},
			order = 100,
		},
	},
}

-----------------------------------------------------------------------
-- Addon initialization
-----------------------------------------------------------------------
Violation = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceComm-2.0", "FuBarPlugin-2.0", "Parser-3.0")
local Violation = Violation

Violation.hasIcon = "Interface\\Icons\\Spell_ChargePositive"

function Violation:OnInitialize()
	local revision = tonumber(("$Revision: 28352 $"):sub(12, -3)) or 1
	if not self.version then self.version = "1.0" end
	self.version = self.version .. "." .. revision
	self.revision = revision

	_G.VIOLATION_VERSION = self.version
	_G.VIOLATION_REVISION = self.revision

	self:RegisterDB("ViolationDB")
	self:RegisterDefaults("profile", {
		gainCombat = nil,
		leaveCombat = nil,
		leaderReset = true,
	})

	self.OnMenuRequest = options

	self:RegisterChatCommand({ "/violation" }, self.OnMenuRequest, "VIOLATION")

	playerName = UnitName("player")
end

function Violation:OnEnable()
	self:Reset()

	self:RegisterParserEvent({
		eventType = "Damage",
		sourceName = "player",
	}, "OnCombatEvent")

	self:SetCommPrefix("Violation")
	self:SetDefaultCommPriority("BULK")
	self:RegisterComm("Violation", "GROUP", "OnComm")

	self:ScheduleRepeatingEvent("DamageBroadcast", self.Broadcast, 5, self)

	db = self.db.profile

	if db.gainCombat then
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "Reset")
	end

	if db.leaveCombat then
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "Reset")
	end
end

function Violation:Reset()
	data = {
		[playerName] = 0
	}
end

-----------------------------------------------------------------------
-- UI
-----------------------------------------------------------------------

local function Sorter(a, b)
	return a[2] > b[2]
end

local tbl = {}
function Violation:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		"columns", 3,
		"child_justify", "LEFT",
		"child_justify2", "LEFT",
		"child_justify3", "RIGHT",
		"child_textR", 1,
		"child_textG", 1,
		"child_textB", 0,
		"child_text3R", 1,
		"child_text3G", 1,
		"child_text3B", 1
	)
	for k, v in pairs(data) do
		if type(v) == "number" and v > 0 then
			table.insert(tbl, {k, v})
		end
	end
	table.sort(tbl, Sorter)

	local playerShown = nil
	for i, v in ipairs(tbl) do
		if i > 10 then break end -- Only show the top 10.
		cat:AddLine("text", tostring(i) .. ".", "text2", coloredNames[v[1]], "text3", tostring(v[2]))
		if v[1] == playerName then playerShown = true end
	end

	if not playerShown and data[playerName] > 0 then
		cat:AddLine("text", "?", "text2", coloredNames[playerName], "text3", tostring(data[playerName]))
	end

	-- Cleanup.
	for i, v in ipairs(tbl) do
		tbl[i] = nil
	end
end

-----------------------------------------------------------------------
-- Data broadcasting and recieving
-----------------------------------------------------------------------
function Violation:Broadcast()
	if dirty == 0 then return end
	dirty = 0
	self:SendCommMessage("GROUP", data[playerName])
end

function Violation:ResetGroup()
	if not IsRaidLeader() or not IsRaidOfficer() or not IsPartyLeader() then
		error("Only the group leader or an officer can do that.")
		return
	end

	self:SendCommMessage("GROUP", "LeaderReset")
end

function Violation:OnComm(prefix, sender, distribution, input)
	if distribution ~= "GROUP" then return end
	if tonumber(input) then
		data[sender] = tonumber(input)
		self:ScheduleEvent("TooltipUpdate", self.UpdateTooltip, 5, self)
	elseif type(input) == "string" then
		if input == "LeaderReset" then
			local obj = roster:GetUnitObjectFromName(sender)
			if obj.rank > 0 then
				self:Print(string.format(L["Resetting data as requested by %s."], sender))
				self:Reset()
			end
		end
	end
end

-----------------------------------------------------------------------
-- Combat log parsing
-----------------------------------------------------------------------
function Violation:OnCombatEvent(t)
	assert(t.sourceName == "player", "Not supposed to get combat events from people except ourselves.")
	if t.amount then
		data[playerName] = data[playerName] + t.amount
		dirty = dirty + t.amount
		self:UpdateTooltip()
	end
end

